#include "grafo.h"

#include <assert.h>

//! Crea un grafo

/*!
    Crea un grafo de <tt>n</tt> nodos.

    \param n Nmero de nodos. Debe ser mayor que 0.
 */
Grafo::Grafo( int n )
{
    assert( n>0 );
    inicializar( n );
}

//! Inicializa un grafo.

/*!
    Inicializa la matriz de conexiones de un grafo.

    \param n Nmero de nodos. Debe ser mayor que 0.
 */
void Grafo::inicializar( int n )
{
    assert( n>0 );
    _numNodos = n;
    _matriz = new TipoPeso*[numNodos()];
    for ( int i=0; i<numNodos(); i++ ) {
        _matriz[i] = new TipoPeso[numNodos()];
    }

    // Inicialmente no hay conexiones
    for ( int i=0; i<numNodos(); i++) {
        for ( int j=0; j<numNodos(); j++) {
            _matriz[i][j] = infinito;
        }
    }
}

//! Destruye un grafo

/*!
    Destruye un grafo.
 */
Grafo::~Grafo( void )
{
    liberar( );
}

//! Libera la matriz

/*!
    Libera la matriz de conexiones del grafo.
 */
void Grafo::liberar( void )
{
    for (int i=0;i<numNodos();i++) {
        delete []_matriz[i];
    }
    delete []_matriz;
}

//! Copiar pesos de otro grafo.

/*!
    Copia los pesos del grafo <tt>otro</tt> en el actual.

    \param otro Origen para la copia. Se requiere que
        this->numNodos()==otro->numNodos()
 */
void Grafo::copiaPesos( Grafo *otro )
{
    assert( numNodos() == otro->numNodos() );

    for ( int i=0; i<numNodos(); i++) {
        for ( int j=0; j<numNodos(); j++) {
            peso( i, j, otro->peso( i, j ) );
        }
    }
}

//! Nmero de nodos del grafo

/*!
    Devuelve el nmero de nodos del grafo.

    \returns Nmero de nodos del grafo. Se asegura que es mayor que 0.
 */
int Grafo::numNodos( void ) const
{
    return _numNodos;
}

//! Peso de un arco

/*!
    Devuelve el peso del arco que conecta los nodos
    <tt>i</tt> y <tt>j</tt>.

    \param i Origen del arco.
    \param j Destino del arco.
    \returns Peso del arco.
 */
Grafo::TipoPeso Grafo::peso( int i, int j ) const
{
    assert( i>=0 && i<numNodos() );
    assert( j>=0 && j<numNodos() );

    return _matriz[i][j];
}

//! Poda arcos mayores que un umbral.

/*!
    Cambia por infinito los pesos de los arcos cuyo
    peso actual sea mayor que <tt>umbral</tt>

    \param umbral Umbral de la poda.
 */
void Grafo::podarMayores( TipoPeso umbral )
{
    TipoPeso p;

    for ( int i=0; i<numNodos(); i++ ) {
        for ( int j=0; j<numNodos(); j++ ) {
            p = peso( i, j );
            if ( p>umbral ) {
                peso( i, j, infinito );
            }
        }
    }
}
//! Crea la potencia enesima de la matriz de conexiones 
/*!
    Cambia la matriz de conexiones por ella misma 
    elevada a un determinado numero <tt>n</tt>

    \param n numero al que vamos a elevar la matriz.
 */
 
/* Lo que hacemos es que al elevar la potencia obtenemos si existe o no
un camino para ir de un nodo a otro */
void Grafo::potencia(int n)
{
  int i, grado, j ,k, l,m, acumulador=0;  //Aqui guardaremos el nuevo, acumulador

  //Reservamos memoria para ambas matrices
  TipoPeso **producto=new TipoPeso*[numNodos()];
  for ( i=0; i<numNodos(); i++ )
    {
      producto[i] = new TipoPeso[numNodos()];
    }

  TipoPeso ** aux=new TipoPeso*[numNodos()];
  for (i=0; i<numNodos(); i++ ) 
    {
      aux[i] = new TipoPeso[numNodos()];
    }

  // copiamos los valores de la matriz actual a una matriz binaria
  for ( l=0; l<numNodos(); l++ ) 
    {
      for ( m=0; m<numNodos(); m++ )
	{
          if (_matriz[l][m]== infinito)
	    {
	      _matriz[l][m]=aux[l][m]=0;
	    }
	  else
	    {
	      _matriz[l][m]=aux[l][m]=1;
	    }
	}
    }

  // realizamos el producto.
  for(grado=0; grado<n-1; grado++)
    { 
      for ( i=0; i<numNodos(); i++ )
	{
        for ( j=0; j<numNodos(); j++ )
	  {
	    for ( k=0; k<numNodos(); k++ )
	      {
	        // En acumulador guardamos el producto de fila y columna.
		acumulador+=peso(i,k)*aux[k][j];
	      }
	    // guardamos el producto.
	    producto[i][j]=acumulador;
	    acumulador=0; // inicializamos el acumulador a 0
	  }
	}
      // almacenamos el producto parcial
      for ( l=0; l<numNodos(); l++ )
	{
	  for ( m=0; m<numNodos(); m++ ) 
	    {
              _matriz[l][m]=producto[l][m];
	    }
	}
    }
  /* Una vez realizado el producto  con la matriz binaria, ponemos
     infinito en vez de 0 */
  for ( l=0; l<numNodos(); l++ )
    {
      for ( m=0; m<numNodos(); m++ ) 
	{
	  if (_matriz[l][m]==0)
	    {
	      _matriz[l][m]=infinito;
	    }
	}
    }
  delete producto;
  delete aux;
}


int ** Grafo::floyd(int origen)
{
  int i,j,k;
  
// Vamos a hacerlo como en el libro.
  TipoPeso **A=new TipoPeso*[numNodos()];
  for (i=0; i<numNodos(); i++ )
    {
      A[i] = new TipoPeso[numNodos()];
    }
// Esta matriz P guarda los nodos intermedios
    TipoPeso **P=new TipoPeso*[numNodos()];
  for (i=0; i<numNodos(); i++ )
    {
      P[i] = new TipoPeso[numNodos()];
    }

  
  /* inicializamos las matrices*/
  
  for ( i=0; i<numNodos(); i++ ) 
    {
      for (j=0; j<numNodos(); j++ )
	{
	  A[i][j]=peso(i,j); // lo mismo que _matriz[i][j];
	  P[i][j]=0; // inicialmente todo 0
	}
      // ponemos la diagonal principal a 0
      A[i][i]=0;
    }
// calculamos los caminos minimos y los nudos intermedios los guardamos en P  
  for(k=0; k<numNodos(); k++)
    { 
      for ( i=0; i<numNodos(); i++ )
	{
	  for (j=0; j<numNodos(); j++ )
	    {
	      if (A[i][j] >(A[i][k]+A[k][j]))
		{
		  A[i][j]=A[i][k]+A[k][j];
		  // nudo medio intermedio que se acorto por ultima vez
		  P[i][j]=k;
		}
	    }
	}  
    }
  /* ponemos a infinito todos los valores que sean 0 y copiamos A a la matriz */
  for ( i=0; i<numNodos(); i++ ) 
    {
      for ( j=0; j<numNodos(); j++ ) 
	{
	  if (A[i][j]==0 || i!=origen)
	    {
	    _matriz[i][j]=infinito;
	    }
	   else
	   	{
	   	_matriz[i][j]=A[i][j];
		}
	}
    }
// ha finalizado debemos devolver P
return P;
}

//

void Grafo::caminofloyd(int **P, int i, int j, QString &cadena)
{
QString valor;
int k;
k=P[i][j];
if(k!=0)
	{
//	caminofloyd(P,i,k, cadena);
	valor.setNum(P[i][j]);
	cadena=cadena+","+valor;
//	caminofloyd(P,k,j, cadena);
	}
}


/*
void Grafo::dijkstra();

*/
