#include "introducirpesosdlg.h"
#include "grafo.h"
#include "grafodirigido.h"
#include "grafonodirigido.h"

#include <qtable.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qmessagebox.h>

#include <assert.h>

//! Crea el cuadro de dilogo

/*!
    Crea un cuadro de dilogo Introducir pesos.

    \param g Grafo a editar.
    \param parent Widget padre.
    \param name Nombre del widget.
    \param modal Se mostrar el cuadro como modal?
    \param f Opciones de creacin del widget.
 */
IntroducirPesosDlg::IntroducirPesosDlg( Grafo *g, QWidget *parent = 0,
    const char *name = 0, bool modal = FALSE,
    WFlags f = 0 ):QDialog( parent, name, modal, f )
{
    assert( g!=NULL );
    _grafo = g;

    // Creamos la tabla y la inicializamos con los pesos del grafo
    QVBoxLayout *vb = new QVBoxLayout( this, 8 );

    _tabla=new QTable( _grafo->numNodos(), _grafo->numNodos(), this );
    vb->addWidget( _tabla );

    for ( int i=0; i<_grafo->numNodos(); i++ ) {
        for ( int j=0; j<_grafo->numNodos(); j++ ) {
            Grafo::TipoPeso peso = _grafo->peso( i, j );
            if ( peso == Grafo::infinito ) {
                _tabla->setText( i, j, "Infinito" );
            } else {
                _tabla->setText( i, j, QString::number( peso ) );
            }
        }
    }

    /*
     * En los grafos no dirigidos hay que asegurar que
     * peso[i][j] == peso[j][i]
     */
    GrafoNoDirigido *ndg=dynamic_cast<GrafoNoDirigido *>(_grafo);
    if ( ndg!=NULL ) {
        connect( _tabla, SIGNAL( valueChanged( int, int ) ), this,
            SLOT( retocarPesosNoDirigidos( int , int ) ) );
    }

    // Creamos los botones Aceptar y Cancelar
    QHBoxLayout *hb= new QHBoxLayout( vb );

    QPushButton *aceptar = new QPushButton( "Aceptar", this );
    hb->addWidget( aceptar );
    aceptar->setDefault( true );
    connect( aceptar, SIGNAL( clicked() ), this, SLOT( accept() ) );

    QPushButton *cancelar = new QPushButton( "Cancelar", this );
    hb->addWidget( cancelar );
    connect( cancelar, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

//! Destruye el cuadro de dilogo

/*!
    Destructor del cuadro de dilogo
 */
IntroducirPesosDlg::~IntroducirPesosDlg()
{

}

//! El usuario acepta los datos.

/*!
    Cdigo a ejecutar cuando el usuario acepta las modificaciones
    realizadas a los pesos.
 */
void IntroducirPesosDlg::accept( void )
{
    bool esNumero;
    QString mensaje="Valor no vlido en fila %1 columna%2";

    // Comprobamos que todos los valores son vlidos
    for ( int i=0; i<_grafo->numNodos(); i++ ) {
        for ( int j=0; j<_grafo->numNodos(); j++ ) {
            QString valor = _tabla->text( i, j );

            // Depende de que Grafo::TipoPeso sea int
            valor.toInt( &esNumero );
            if ( !esNumero && valor.lower() != "infinito" ) {
                QMessageBox::critical( this, 0, mensaje.arg(i+1).arg(j+1) );
                return;
            }
        }
    }

    // Volcamos los valores al grafo
    for ( int i=0; i<_grafo->numNodos(); i++ ) {
        for ( int j=0; j<_grafo->numNodos(); j++ ) {
            QString valor = _tabla->text( i, j );
            if ( valor.lower() == "infinito" ) {
                _grafo->peso( i, j, Grafo::infinito );
            } else {
                _grafo->peso( i, j, valor.toInt() );
            }
        }
    }
    done( Accepted );
}

//! Asegura que un grafo no dirigido lo sigue siendo

/*!
    Este slot asegura que un grafo no dirigido lo sigue
    siendo al editar el usuario los pesos. Para ello, asegura
    que peso[fila][columna] == peso[columna][fila]. Se ejecuta
    cada vez que el usuario modifica un peso en un grafo
    no dirigido.

    \param fila Fila del peso que ha cambiado el usuario.
    \param columna Columna del peso que ha cambiado el usuario.
 */
void IntroducirPesosDlg::retocarPesosNoDirigidos( int fila, int columna )
{
    /*
     * En los grafos no dirigidos hay que asegurar que
     * peso[i][j] == peso[j][i]
     */
    if ( fila!=columna ) {
        _tabla->setText( columna, fila, _tabla->text( fila, columna ) );
    }
}
