#include "vistagrafo.h"
#include "grafo.h"
#include "grafodirigido.h"
#include "grafonodirigido.h"

#include <qwidget.h>
#include <qpushbutton.h>
#include <qpen.h>
#include <qrect.h>
#include <qpainter.h>
#include <assert.h>
#include <math.h>


//! Widget que representa un nodo

/*!
    Widget para representar los nodos del grafo.
 */
class VistaGrafo::NodoGrafo:public QWidget {
    public:
        NodoGrafo( int n, VistaGrafo *parent, const char *name=0, WFlags f=0);
        static const int radio=25;

        bool seleccionado();
        void seleccionado(bool estado);

        //! Nmero de nodo.
        /*!
            Devuelve el nmero del nodo.

            \returns Nmero del nodo.
         */
        int numero() {return _numero;};
    protected:
        virtual void mousePressEvent( QMouseEvent *ev );
        virtual void paintEvent ( QPaintEvent *ev);
        virtual void enterEvent(QEvent *e);
        virtual void mouseReleaseEvent(QMouseEvent *ev);
        virtual void mouseMoveEvent( QMouseEvent * );

    private:
        //! Nmero del nodo

        /*!
            Nmero del nodo.
         */
        int _numero;

        //! Se est moviendo el nodo?

        /*!
            Indica si el nodo est siendo arrastrado
         */
        bool _moviendo;

        //! Est seleccionado el nodo?

        /*!
            Indica si el nodo est seleccionado.
         */
        bool _seleccionado;

        //! Etiqueta del nodo.

        /*!
            Etiqueta a dibujar en el nodo (el nmero)
         */
        QString _etiqueta;

        //! Vista padre

        /*!
            Vista de la que depende el widget.
         */
        VistaGrafo *_vista;
};

//! Construye un widget nodo

/*!
    Construye un widget para representar a un nodo de un grafo.

    \param n Nmero del nodo.
    \param parent Vista padre.
    \param name Nombre del widget.
    \param f Opciones de creacin.
 */
VistaGrafo::NodoGrafo::NodoGrafo(int n, VistaGrafo *parent,
    const char *name=0,WFlags f=0):QWidget( parent, name, f),
    _etiqueta( QString::number( n ) )
{
    _numero=n;
    _moviendo=false;
    _seleccionado=false;
    _vista = parent;

    setFixedSize( 2*radio, 2*radio );
}

//! Est seleccionado el nodo?

/*!
    Estado de seleccin del nodo.

    \returns <tt>true</tt> si el nodo est seleccionado,
        <tt>false</tt> en otro caso.
 */
bool VistaGrafo::NodoGrafo::seleccionado()
{
    return _seleccionado;
}

//! Cambio del estado de seleccin

/*!
    Cambia el estado de seleccin del nodo a <tt>estado</tt>.

    \param estado Nuevo estado de seleccin.
 */
void VistaGrafo::NodoGrafo::seleccionado(bool estado)
{
    _seleccionado=estado;
    update();
    _vista->viewport()->update();
}

//! Un botn del ratn ha sido presionado

/*!
    Gestin para eventos de presionado de botones de ratn.

    \param ev Evento.
 */
void VistaGrafo::NodoGrafo::mousePressEvent( QMouseEvent *ev )
{
    if ( ev->button() == LeftButton ) {
        setMouseTracking( true );
        _moviendo=true;
        seleccionado( !seleccionado() );
        // Actualizamos la vista porque cambia el estado de seleccion
        _vista->viewport()->update();
        update();
    }
}

//! Un botn del ratn ha sido soltado

/*!
    Gestin para eventos de soltado de botones de ratn.

    \param ev Evento.
 */
void VistaGrafo::NodoGrafo::mouseReleaseEvent(QMouseEvent *ev)
{
    (void) ev;
    _moviendo=false;
}

//! El ratn se ha movido

/*!
    Gestin para eventos de movimiento de ratn.

    \param ev Evento.
 */
void VistaGrafo::NodoGrafo::mouseMoveEvent( QMouseEvent *ev )
{
    if ( _moviendo ) {
        int x,y;

        x=_vista->childX( this );
        y=_vista->childY( this );

        x+=ev->x();
        y+=ev->y();

        _vista->moveChild( this, x, y );
        _vista->viewport()->update();
    }
}

//! Dibujado del widget

/*!
    Gestin del evento de dibujado del widget.

    \param ev Evento.
 */
void VistaGrafo::NodoGrafo::paintEvent ( QPaintEvent *ev )
{
    (void) ev;
    QPainter painter( this );

    if ( seleccionado() ) {
        painter.setPen( QPen( black, 3 ) );
        QFont f = painter.font();
        f.setWeight( QFont::Bold );
        painter.setFont( f );
    } else {
        painter.setPen( QPen( black, 1 ) );
        QFont f = painter.font();
        f.setWeight( QFont::Normal );
        painter.setFont( f );
    }

    painter.setBrush( QBrush( qRgb( 0xf7, 0xef, 0xd6 ), Qt::SolidPattern ) );

    QFontMetrics fm = painter.fontMetrics();

    int altoEtiqueta = fm.ascent();
    int anchoEtiqueta = fm.width( _etiqueta );

    int px = ( width() - anchoEtiqueta )/2;
    int py = ( height() + altoEtiqueta )/2;

    painter.drawRect( rect() );
    painter.drawText( px, py, _etiqueta );
}

//! El puntero del ratn entra en el widget

/*!
    Gestin del evento de entrada de ratn.

    \param ev Evento.
 */
void VistaGrafo::NodoGrafo::enterEvent(QEvent *ev)
{
    (void) ev;
    setCursor(QCursor(PointingHandCursor));
    update();
}

///////////////////////////////////////////////////////////////

//! Crea la vista

/*!
    Crea una vista de grafo.

    \param g Grafo.
    \param parent Widget padre.
    \param name Nombre del widget.
    \param f Opciones de creacin.
 */
VistaGrafo::VistaGrafo ( Grafo *g, QWidget * parent = 0,
    const char * name = 0, WFlags f = 0 ):
        QScrollView( parent, name, f ),
        _listaNodos( g->numNodos() )
{
    assert( g!=NULL );
    _grafo = g;
    crearNodos();

    GrafoNoDirigido *ndg=dynamic_cast<GrafoNoDirigido *>(_grafo);
    _noDirigido = (ndg!=NULL);
}

//! Destruye la vista

/*!
    Destruye la vista.
 */
VistaGrafo::~VistaGrafo()
{

}

//! Dibuja el contenido de la vista.

/*!
    Dibuja el contenido de la vista que estn en el rectngulo
    de recorte indicado.

    \param p Objeto dibujante.
    \param clipx Coordenada x de la esquina superior izquierda del
        rectngulo de clipping.
    \param clipy Coordenada y de la esquina superior izquierda del
        rectngulo de clipping.
    \param clipw Ancho del rectngulo de clipping.
    \param cliph Alto del rectngulo de clipping.
 */
void VistaGrafo::drawContents ( QPainter * p, int clipx, int clipy, int clipw,
    int cliph )
{
    int n=_grafo->numNodos();
    int i,j;
    Grafo::TipoPeso peso;

    QBrush brocha( qRgb( 0xf7, 0xef, 0xd6 ), Qt::SolidPattern );
    QRect recorte( clipx, clipy, clipw, cliph );
    p->fillRect( recorte, brocha );

    if ( _noDirigido ) {
        for ( i=0; i<n; i++ ) {
            for ( j=i; j<n; j++ ) {
                peso = _grafo->peso( i, j );
                if ( peso!=Grafo::infinito ) {
                    dibujarArcoNoDirigido( p, i, j );
                }
            }
        }
    } else {
        for ( i=0; i<n; i++ ) {
            for ( j=0; j<n; j++ ) {
                peso = _grafo->peso( i, j );
                if ( peso!=Grafo::infinito ) {
                    dibujarArcoDirigido( p, i, j );
                }
            }
        }
    }

}

//! Dibuja un arco no dirigido

/*!
    Dibuja un arco en un grafo no dirigido.

    \param p Objeto dibujante.
    \param n1 Primer nodo.
    \param n2 Segundo nodo.
 */
void VistaGrafo::dibujarArcoNoDirigido( QPainter *p, int n1, int n2 )
{
    NodoGrafo *wa, *wb;

    wa = _listaNodos[n1];
    wb = _listaNodos[n2];

    if ( wa->seleccionado() && wb->seleccionado() ) {
        p->setPen( QPen( Qt::black, 2 ) );
        QFont f=p->font();
        f.setWeight( QFont::Bold );
        p->setFont( f) ;
    } else {
        p->setPen( QPen( Qt::black, 1 ) );
        QFont f = p->font();
        f.setWeight( QFont::Normal );
        p->setFont( f );
    }

    if ( n1 == n2 ) {
        int waX = childX(wa) + 5;
        int waXOffset = wa->width() - 10;
        int waY = childY(wa) - 30;

        p -> drawArc(waX, waY, waXOffset, 60, -180*16, -180*16);
        p->drawText( waX+waXOffset/2, waY-2,
            QString::number( _grafo->peso( n1, n2 ) ) );
    } else {
        int xa, xb, ya, yb;
        int mx, my;

        xa = (int) ( childX(wa) + wa->width() * 0.5 );
        xb = (int) ( childX(wb) + wb->width() * 0.5 );
        ya = (int) ( childY(wa) + wa->height() * 0.5 );
        yb = (int) ( childY(wb) + wb->height() * 0.5 );
        QPoint point = widgetIntersectPoint(xa, ya, xb, yb, wb);

        p -> drawLine(xa, ya, point.x(), point.y());

        if ( xa>point.x() )
            mx=point.x()+(xa-point.x())/2;
        else
            mx=xa+(point.x()-xa)/2;

        if ( ya>point.y() )
            my=point.y()+(ya-point.y())/2;
        else
            my=ya+(point.y()-ya)/2;

        QPoint puntoMedio( mx, my );
        p->drawText( puntoMedio, QString::number( _grafo->peso( n1, n2 ) ) );
    }
}

//! Dibuja un arco dirigido

/*!
    Dibuja un arco en un grafo dirigido.

    \param p Objeto dibujante.
    \param n1 Nodo origen.
    \param n2 Nodo destino.
 */
void VistaGrafo::dibujarArcoDirigido( QPainter *p, int n1, int n2 )
{
    NodoGrafo *wa, *wb;

    wa = _listaNodos[n1];
    wb = _listaNodos[n2];

    if ( wa->seleccionado() && wb->seleccionado() ) {
        p->setPen( QPen( Qt::black, 3 ) );
        QFont f=p->font();
        f.setWeight( QFont::Bold );
        p->setFont( f) ;
    } else {
        p->setPen( QPen( Qt::black, 1 ) );
        QFont f = p->font();
        f.setWeight( QFont::Normal );
        p->setFont( f );
    }

    if ( n1 == n2 ) {
        int waX = childX(wa) + 5;
        int waXOffset = wa->width() - 10;
        int waY = childY(wa) - 30;
        p -> drawArc(waX, waY, waXOffset, 60, -180*16, -180*16);

        p->drawText( waX+waXOffset/2, waY-2,
            QString::number( _grafo->peso( n1, n2 ) ) );
    } else {
        int xa, xb, ya, yb;
        int mx, my;
        float correccion;

        if ( n2>n1 )
            correccion=0.75;
        else
            correccion=0.25;

        xa = (int) ( childX(wa) + wa->width() * correccion );
        xb = (int) ( childX(wb) + wb->width() * correccion );
        ya = (int) ( childY(wa) + wa->height() * correccion );
        yb = (int) ( childY(wb) + wb->height() * correccion );

        /*
            calculate all the values needed to draw the lines,
            arrow & diamond heads
         */
        QPoint point = widgetIntersectPoint(xa, ya, xb, yb, wb);

	    int halfLength = 10;
	    double slope = atan2(yb - ya, xb - xa);//slope of line
	    double arrowSlope = slope + atan(1);
	    QPoint arrowPointA(point.x() - halfLength * cos(arrowSlope),
            point.y() - halfLength * sin(arrowSlope));
	    arrowSlope = slope - atan(1);
	    QPoint arrowPointB(point.x() - halfLength * cos(arrowSlope),
            point.y() - halfLength * sin(arrowSlope));

        p -> drawLine(xa, ya, point.x(), point.y());
        p -> drawLine(point.x(), point.y(), arrowPointA.x(), arrowPointA.y());
        p -> drawLine(point.x(), point.y(), arrowPointB.x(), arrowPointB.y());

        if ( xa>point.x() )
            mx=point.x()+(xa-point.x())/2;
        else
            mx=xa+(point.x()-xa)/2;

        if ( ya>point.y() )
            my=point.y()+(ya-point.y())/2;
        else
            my=ya+(point.y()-ya)/2;

        QPoint puntoMedio( mx, my );
        p->drawText( puntoMedio, QString::number( _grafo->peso( n1, n2 ) ) );
    }
}

//! Crea los nodos de la vista

/*!
    Crea los widgets hijos que representan a los nodos.
 */
void VistaGrafo::crearNodos( void )
{
    int n=_grafo->numNodos();
    float radio=n*NodoGrafo::radio;
    float paso=2*M_PI/n;
    int r=NodoGrafo::radio;
    int i;
    int maxX=0, maxY=0;
    float alfa;

    for ( i=0, alfa=0.0; i<n; i++, alfa+=paso ) {
        float x= radio+NodoGrafo::radio+radio*cos( alfa )-r;
        float y= radio+NodoGrafo::radio+radio*sin( alfa )-r;

        NodoGrafo *nuevo = new NodoGrafo( i, this );
        addChild( nuevo, x, y );

        if ( x > maxX )
            maxX = (int)x;

        if ( y > maxY )
            maxY = (int)y;
        _listaNodos[i]=nuevo;
    }

    resizeContents( maxX+3*NodoGrafo::radio, maxY+3*NodoGrafo::radio );
}

//! Clculo de los puntos extremo de los arcos.

/*!
    Tomado de "UML Modeller" v1.0.3
    \author (c) Paul Hensgen 2001
 */
QPoint VistaGrafo::widgetIntersectPoint(int xa, int ya, int xb, int yb,
    QWidget *wb)
{
    //find where line intersects widget b
    int wx = childX(wb);
    int wy = childY(wb);
    QPoint pa(xa, ya), pb(xb, yb), point;
    double slope = getSlope(pa, pb);
    double angle = getSlope(pb, QPoint(wx, wy) );
    angle = angle > 0 ? angle : angle * -1;
    //see if line pa - pb is horiontal or vertical
    //if so save time with calculations, as we can do easier.
    if (slope == 0) {
        point.setY(ya);
		if(xa < xb)
			point.setX(wx);
		else
			point.setX(wx + wb -> width());
		return point;
	}
	else if(slope == 1000) {
		point.setX(xa);
		if(ya < yb)
			point.setY(wy);
		else
			point.setY(wy + wb -> height());
		return point;
	}

    //if we are on left or right swap x,y around
    //do this as algorithm bums out as it can't handle infinite slope well.

    if(yb < ya) {
   	    if( (slope < -angle /*&& xb > xa*/) ||
                (slope > angle /*&& xb < xa*/) ) { // Bottom
      	    QPoint w1(wx, wy + wb -> height());
      	    QPoint w2(wx + wb -> width(), wy + wb -> height());
      	    point = intersectPoint(w1, w2, pa, pb);
        } else if(/*slope > 0 && */xb < xa) { // Right
      	    QPoint w1( wy, wx + wb -> width());
      	    QPoint w2(wy + wb -> height(),wx + wb -> width());
      	    pa.setX(ya);
      	    pa.setY(xa);
      	    pb.setX(yb);
      	    pb.setY(xb);
      	    point = intersectPoint(w1, w2, pa, pb);
      	    int x = point.x();
      	    int y= point.y();
      	    point.setX(y);
      	    point.setY(x);
        } else { //Left
      	    QPoint w1(wy, wx);
      	    QPoint w2(wy + wb -> height(), wx);
      	    pa.setX(ya);
      	    pa.setY(xa);
      	    pb.setX(yb);
      	    pb.setY(xb);
      	    point = intersectPoint(w1, w2, pa, pb);
      	    int x = point.x();
      	    int y= point.y();
      	    point.setX(y);
      	    point.setY(x);
        }
    } else if(yb > ya ) {
   	    if( (slope < -angle /*&& xb < xa*/) ||
                (slope > angle /*&& xb > xa*/)) { //top
   	  	    QPoint w1(wx, wy);
      	    QPoint w2(wx + wb -> width(), wy);
      	    point = intersectPoint(w1, w2, pa, pb);
        } else if(/*slope < 0.3 &&*/ xb > xa) { //left
            QPoint w1(wy, wx);
  	   	    QPoint w2(wy + wb -> height(), wx);
   		    pa.setX(ya);
	   	    pa.setY(xa);
   		    pb.setX(yb);
      	    pb.setY(xb);
      	    point = intersectPoint(w1, w2, pa, pb);
  		    int x = point.x();
      	    int y= point.y();
      	    point.setX(y);
      	    point.setY(x);
        } else { //right
   		    QPoint w1( wy, wx + wb -> width());
      	    QPoint w2(wy + wb -> height(),wx + wb -> width());
      	    pa.setX(ya);
	   	    pa.setY(xa);
   		    pb.setX(yb);
      	    pb.setY(xb);
      	    point = intersectPoint(w1, w2, pa, pb);
      	    int x = point.x();
	   	    int y= point.y();
   		    point.setX(y);
      	    point.setY(x);
        }
    }//end if
    return point;
}

//! Clculo de los puntos extremo de los arcos.

/*!
    Tomado de "UML Modeller" v1.0.3
    \author (c) Paul Hensgen 2001
 */
float VistaGrafo::getSlope(QPoint p1, QPoint p2)
{
	float slope = 0;
	if(p1.y() - p2.y() == 0)
		return 0;
	if(p1.x() - p2.x() == 0)
		return 1000;//should be infinite

	slope = (float)(p1.y() - p2.y()) / (p1.x() - p2.x());
	return slope;
}

//! Clculo de los puntos extremo de los arcos.

/*!
    Tomado de "UML Modeller" v1.0.3
    \author (c) Paul Hensgen 2001
 */
QPoint VistaGrafo::intersectPoint(QPoint p1, QPoint p2, QPoint q1, QPoint q2)
{
	//find slope

	float b1, b2;
	int a1, a2;
	int x, y;

	b1 = getSlope(p1, p2);
	b2 = getSlope(q1, q2);

	a1 = (int) ( p1.y() - b1 * p1.x() );
	a2 = (int) ( q1.y() - b2 * q1.x() );
	x = (int) ( ( -1 * (a1 - a2) ) / (b1- b2) );
	y = (int) ( a1 + (b1 * x) );


	QPoint pq(x, y);
	return pq;
}

//! Seleccin/deseleccin de todos los nodos

/*!
    Cambia el estado de seleccin de todos los nodos
    del grafo.

    \param estado Nuevo estado de seleccin.
 */
void VistaGrafo::seleccionarTodo( bool estado )
{
    vector<NodoGrafo *>::iterator i;

    for ( i=_listaNodos.begin(); i!=_listaNodos.end(); i++ ) {
        NodoGrafo *n=*i;
        n->seleccionado( estado );
        viewport()->update();
    }
}

//! El grafo ha sido modificado

/*!
    Se indica a la vista que se ha modificado el grafo.
 */
void VistaGrafo::grafoModificado( void )
{
    viewport()->update();
}
